
% clear all;
warning off MATLAB:log:LogOfZero;

%%%run parameters:
M = 40;

num_vars = 4;
parameters.hyperparameter=[0.5 0.5];
parameters.cpt_model='beta';
parameters.config='small_memory';
% parameters.cpt_model='noisy-or';
%   parameters.leak=[8 20];
%   parameters.strength=[100 10];
num_of_systems = 50;
min_samples_per_system=2;
max_samples_per_system=2000;
grid_steps=25; %approximate number of grid steps...

% UF={[1 0 0 0; 0 1 0 0], [0 0 0 0; 0 1 0 0]};
% theory_compliance_per_system=[ones(1,25) 2.*ones(1,25)];

%two unary features, one coincides with action set, other doesn't.
RF=[]; %relational variable feature, unused for now.

%parameters that determine how the data is generated and what stats are
%kept:
action_set_ind=2; %first variable will be action in sampled systems.
% theories_of_interest=[1 8]; %null and gold.
% sel_theory_index = 8; %CBN with known actions =UF.


%%%get ready:
parameters.score_data_at=unique(ceil(logspace(log10(min_samples_per_system),log10(max_samples_per_system),grid_steps)));
offset=(0:length(parameters.score_data_at)-1)';
slice= ones(1,num_of_systems);
data_sets_per_system= repmat(offset,1,length(slice)) + repmat(slice,length(offset),1);
num_of_data_sets=size(data_sets_per_system,1);
num_of_samples = max(parameters.score_data_at);
total_samples = sum(parameters.score_data_at(data_sets_per_system),2);

num_theories_of_interest=length(theories_of_interest);

%%%load (or build) dags:
dags = bootstrap_build_all_DAGs(num_vars,'load_saved');

%%%build theory set and P(model|theory):
[theory_compliance theory_set] = build_theories(num_vars,dags,UF,RF);%, 'load_saved');

%%%do the runs:
rankt = zeros(num_of_data_sets,num_theories_of_interest,M);
rankd = zeros(num_of_systems,num_of_data_sets,1+num_theories_of_interest,M);
postd = zeros(num_of_systems,num_of_data_sets,1+num_theories_of_interest,M);
theory_log_prob = zeros(size(theory_set,1),num_of_data_sets,M);
for p=1:M
    tic
    display('sampling data...');
    
    %sample dags, cpts, and data for systems:
    [systems_and_samples, system_dags, CPT] = ...
        forward_all(parameters, num_vars, num_of_systems,num_of_samples, dags, theory_compliance, sel_theory_index, action_set_ind);

    toc
    display('scoring dags...');
    
    %generate the (bottom-up) scores P(data|dag):
    scores = score_all(dags, systems_and_samples, parameters);
    
    toc
    display('computing theory and dag probabilities...');
    
    %compute all the other probabilities and rank:
    [rankd(:,:,:,p) rankt(:,:,p) theory_log_prob(:,:,p) postd(:,:,:,p)] = ...
        score_and_rank(dags, system_dags, theory_set, scores, theory_compliance, data_sets_per_system, theories_of_interest, parameters, theory_compliance_per_system);

    toc    
end


%plot the learning curves for systems: no-theory, gold-standard, learned-theory (marginal).
%export with size 8x6in.
figure;
loglog( total_samples, ...  
[ ... %squeeze(mean(rankt(:,2,:),3))'; ...
    squeeze(mean(mean(rankd(:,:,1,:),4),1)); ...
    squeeze(mean(mean(rankd(:,:,2,:),4),1)); ...
    squeeze(mean(mean(rankd(:,:,3,:),4),1)); ...
    ], 'LineWidth', 2);
xlim([min(total_samples) max(total_samples)]);
set(gca,'FontSize', 16, 'FontName', 'Times');
% legend({... %'theory', ...
%                   'system, no theory', ...
%                   'system, correct theory',...
%                   'system, learned theory'}, ...
%                   'FontSize', 16, 'FontName', 'Times')
legend({'no theory', ...
                  'correct theory',...
                  'learned theory'}, ...
                  'FontSize', 16, 'FontName', 'Times');
xlabel('Samples seen (all systems)', 'FontSize', 20, 'FontName', 'Times');
ylabel('Mean rank of correct causal structure', 'FontSize', 20, 'FontName', 'Times');


%plot the mean theory rank, with 5th and 95th percentiles:
figure;
loglog(total_samples, squeeze(mean(rankt(:,2,:),3)),'k',  'LineWidth', 2);
hold on;
sorted=squeeze(sort(rankt(:,2,:),3));
loglog(total_samples, [sorted(:,ceil(size(sorted,2).*0.05))  sorted(:,ceil(size(sorted,2).*0.95))], '--k');
xlim([min(total_samples) max(total_samples)]);
set(gca,'FontSize', 16, 'FontName', 'Times');
xlabel('Samples seen (all systems)', 'FontSize', 20, 'FontName', 'Times');
ylabel('Rank of correct theory', 'FontSize', 20, 'FontName', 'Times');


%plot the law marginals
figure;
for i=1:M
law_marginal_posterior(:,:,i) = exp(theory_log_prob(:,:,i)')*theory_set;
end
semilogx(total_samples, squeeze(mean(law_marginal_posterior,3)));
xlim([min(total_samples) max(total_samples)]);
xlabel('total number of samples');
ylabel('mean marginal probability of law');
L={};
for i=1:size(theory_set,2)
    L{i}=['Law #' num2str(i)];
end
legend(L, 'FontSize', 16, 'FontName', 'Times');


return




%plot the difference between gold-standard and learned-theory (marginal).
%export with size 8x6in.
figure;
loglog( total_samples, ...  
[ squeeze(mean(mean(rankd(:,:,3,:),4),1)) - squeeze(mean(mean(rankd(:,:,2,:),4),1))], 'LineWidth', 2);
xlim([min(total_samples) max(total_samples)]);
set(gca,'FontSize', 16, 'FontName', 'Times');
xlabel('Total # samples', 'FontSize', 20, 'FontName', 'Times');
ylabel('Mean difference in system rank between learned and innate theory', 'FontSize', 20, 'FontName', 'Times');





figure;
for i=1:M
subplot(2,3,i);
law_marginal_posterior(:,:,i) = exp(theory_log_prob(:,:,i)')*theory_set;
semilogx(parameters.score_data_at,exp(theory_log_prob(:,:,i)')*theory_set);
% hold on;
%     loglog(parameters.score_data_at, [...
%     squeeze(mean(rankd_learned_th(:,:,i),1))]./100,'.-');
end




figure;
for i=1:M
    subplot(2,5,i);
    loglog(parameters.score_data_at, [...
    squeeze(mean(rankd_null(:,:,i),1)); ...
    squeeze(mean(rankd_known_learned(:,:,i),1)); ...
    squeeze(mean(rankd_known_noisyobs(:,:,i),1)); ...
    squeeze(mean(rankd_known_obs(:,:,i),1)); ...
    squeeze(mean(rankd_learned_th(:,:,i),1))]);
end





hists=[];
bin_spacing=unique(floor([0 logspace(0,3,100)./2]));
for i=1:M
hists(:,:,i) = histc(rankd_learned_th(:,:,i), bin_spacing);
end
surf(mean(hists,3));





                            