
%%make blessing of abstraction figs:


load big_run.mat;


%  theories_of_interest=[1 4 8 16];
null=1;
gold=3;
learned=size(rankd,3);


%plot the learning curves for systems: no-theory, gold-standard, learned-theory (marginal).
%export with size 8x6in.
figure;
loglog( total_samples, squeeze(mean(mean(rankd(:,:,null,:),4),1)), ':g', ...
        total_samples, squeeze(mean(mean(rankd(:,:,learned,:),4),1)), '-r',...
    total_samples, squeeze(mean(mean(rankd(:,:,gold,:),4),1)), '--b',...
    'LineWidth', 2);
xlim([min(total_samples) max(total_samples)]);
set(gca,'FontSize', 18, 'FontName', 'Times');
% legend({... %'theory', ...
%                   'system, no theory', ...
%                   'system, correct theory',...
%                   'system, learned theory'}, ...
%                   'FontSize', 16, 'FontName', 'Times')
legend({'No theory','Learned theory', 'Correct theory'}, ...
                  'FontSize', 22, 'FontName', 'Times');
xlabel('Total # samples', 'FontSize', 24, 'FontName', 'Times');
ylabel('Mean rank of correct causal structure', 'FontSize', 24, 'FontName', 'Times');
% set(gcf,'PaperPositionMode','auto');
orient landscape;
print -dpdf systemranks.pdf;
% set(gcf, 'Position', [100 100 1000 500]);



%plot the mean theory rank, with percentiles:
figure;
loglog(total_samples, squeeze(mean(rankt(:,gold,:),3)),'m*-',  'LineWidth', 2);
hold on;
sorted=squeeze(sort(rankt(:,gold,:),3));
loglog(total_samples, sorted(:,ceil(size(sorted,2).*0.1)), 'g:',...
        total_samples, sorted(:,ceil(size(sorted,2).*0.5)), 'r:', ...  
        total_samples, sorted(:,ceil(size(sorted,2).*0.9)), 'b:', ...
        'LineWidth', 2);
xlim([min(total_samples) max(total_samples)]);
set(gca,'FontSize', 18, 'FontName', 'Times');
xlabel('Total # samples', 'FontSize', 24, 'FontName', 'Times');
ylabel('Rank of correct theory', 'FontSize', 24, 'FontName', 'Times');
legend({'Mean rank', '10th percentile', '50th percentile', '90th percentile'}, ...
                  'FontSize', 22, 'FontName', 'Times');
orient landscape;
print -dpdf theoryrank.pdf;

% %find the total number of samples (and sample per system) at which the
% %theory is "learned" (rank<2 on average).
% f=find(squeeze(mean(rankt(:,gold,:),3))<2, 1);
% total_samples(f)
% total_samples(f)./num_of_systems



%%plot the mean fraction of systems with correct dag = rank 1,
%and the fraction of runs with true theory = rank 1
figure;
% set(gcf, 'DefaultAxesColorOrder',[0 0 0],...
%       'DefaultAxesLineStyleOrder','-|--|:');
rm=squeeze(mean(mean(rankd(:,:,[null gold learned],:)==1,1),4));
semilogx(total_samples, squeeze(mean(rankt(:,gold,:)==1, 3)), 'm*-', ...
    total_samples, rm(:,1), 'g:', total_samples, rm(:,2), 'b--', total_samples, rm(:,3), 'r-', ...
    'LineWidth', 2);
% hold on;
% semilogx(total_samples, squeeze(mean(rankt(:,2,:)==1, 3)));
set(gca,'FontSize', 18, 'FontName', 'Times');
xlim([min(total_samples) max(total_samples)]);
set(gca,'ytick',[0:0.25:1]);
xlabel('Total # samples', 'FontSize', 24, 'FontName', 'Times');
ylabel('% Learned', 'FontSize', 24, 'FontName', 'Times');
legend({'Theory learning', 'System, no theory', 'System, correct theory', 'System, learned theory'},...
    'FontSize', 22, 'FontName', 'Times', 'Location', 'NorthWest');
orient landscape;
print -dpdf fractioncorrect.pdf;


%plot the law marginals
figure;
for i=1:M
law_marginal_posterior(:,:,i) = exp(theory_log_prob(:,:,i)')*theory_set;
end
set(gcf,...'DefaultAxesColorOrder',[1 0 0;0 1 0;0 0 1],...
      'DefaultAxesLineStyleOrder','-|--|:')
semilogx(total_samples, squeeze(mean(law_marginal_posterior,3)));
set(gca,'FontSize', 18, 'FontName', 'Times');
xlim([min(total_samples) max(total_samples)]);
xlabel('Total # samples', 'FontSize', 24, 'FontName', 'Times');
ylabel('Marginal probability of law', 'FontSize', 24, 'FontName', 'Times');
L={};
for i=1:size(theory_set,2)
    L{i}=['Law #' num2str(i)];
end
legend(L, 'FontSize', 22, 'FontName', 'Times', 'Location', 'East');
orient landscape;
print -dpdf lawmarginals.pdf;





%%find where each law first passes a set probability threshold in each run.
%look at the order of learning across runs.
thresh=0.75;
learning_time=[];
for run=1:M
    for law=1:size(theory_set,2)
        f=find(squeeze(law_marginal_posterior(:,law,run))>thresh, 1);
        if ~isempty(f)
            learning_time(law,run)=f;
        end
    end
end
%learning order of laws 1-3:
[s ind] = sort(learning_time(1:3,:),1);
%fraction of runs in which law 2 was learned last:
law2_last = ind(3,:)==2;
display(['Law 2 is last in '  num2str(mean(law2_last)) ' of runs.']);
display(['of these, Law 3 is first in '  num2str(mean(ind(1,law2_last)==3)) ' of runs, and ']);
display(['  Law 1 is first in '  num2str(mean(ind(1,law2_last)==1)) ' of runs, and']);
display(['  Law 3 is within two steps of law 1 in '  num2str(mean(abs(learning_time(1,law2_last)-learning_time(3,law2_last))<2)) ' of runs.']);


return


%calculate the fraction of systems that have been learned (rank 1) when the
%theory is learned (first reaches rank 1):
[f ind] = find(rankt(:,2,:),1,'first');




%plot the difference between gold-standard and learned-theory (marginal).
%export with size 8x6in.
figure;
loglog( total_samples, ...  
[ squeeze(mean(mean(rankd(:,:,3,:),4),1)) - squeeze(mean(mean(rankd(:,:,2,:),4),1))], 'LineWidth', 2);
xlim([min(total_samples) max(total_samples)]);
set(gca,'FontSize', 16, 'FontName', 'Times');
xlabel('Total # samples', 'FontSize', 20, 'FontName', 'Times');
ylabel('Mean difference in system rank between learned and innate theory', 'FontSize', 20, 'FontName', 'Times');





figure;
for i=1:M
subplot(2,3,i);
law_marginal_posterior(:,:,i) = exp(theory_log_prob(:,:,i)')*theory_set;
semilogx(parameters.score_data_at,exp(theory_log_prob(:,:,i)')*theory_set);
% hold on;
%     loglog(parameters.score_data_at, [...
%     squeeze(mean(rankd_learned_th(:,:,i),1))]./100,'.-');
end




figure;
for i=1:M
    subplot(2,5,i);
    loglog(parameters.score_data_at, [...
    squeeze(mean(rankd_null(:,:,i),1)); ...
    squeeze(mean(rankd_known_learned(:,:,i),1)); ...
    squeeze(mean(rankd_known_noisyobs(:,:,i),1)); ...
    squeeze(mean(rankd_known_obs(:,:,i),1)); ...
    squeeze(mean(rankd_learned_th(:,:,i),1))]);
end





hists=[];
bin_spacing=unique(floor([0 logspace(0,3,100)./2]));
for i=1:M
hists(:,:,i) = histc(rankd_learned_th(:,:,i), bin_spacing);
end
surf(mean(hists,3));


