function [theory_compliance theory_set] = build_theories(N,dags,UF,RF,flag)

%theory_compliance is a matrix (dags X theories X action_sets) of model
%(dag,action_set) probability for each theory (normalized!).
%
%if UF is a cell array, that means compute compiance for several possible
%feature sets. only allow theories that are satisfiable under all the
%theory sets.

if nargin>4
    if strcmp(flag, 'load_saved')
        if exist(['compliance_' num2str(N) '.mat'])==2
            display(['loading theories from theories.mat and theory compliance from ' ['compliance_' num2str(N) '.mat'] '...']);
            load(['compliance_' num2str(N) '.mat']);
            load('theories.mat');
            return;
        end
    end
end

display('building theories...')
tic

if ~iscell(UF)
    return_cell=0;
    UF={UF};
else
    return_cell=1;
end

A_vects=bin_ind2vec((1:2^N)', N);

num_rules=length( law_check(ind2dg(dags(1),N),A_vects(1,:),UF{1},RF) );
num_dags=length(dags);
for u=1:length(UF)
%rule_compliance is a (dags X A_vect X rules) matrix containing the weight
%of each rule on each model (dag,action_set pair).
rule_compliance{u}=zeros(length(dags), 2^N, num_rules);
for d=1:num_dags
    dag=ind2dg(dags(d),N);
    for a=1:2^N
        rule_compliance{u}(d,a,:) = law_check(dag,A_vects(a,:),UF{u},RF);
    end
end
end

%change each rule-compliance matrix into a theory-compliance (weight) matrix 
%(dags X theories X action_sets):
init_theory_set= logical( bin_ind2vec((1:2^num_rules)', num_rules)); %all possible combinations of laws...
max_laws=6; %only theories with fewer than 5 laws...
init_theory_set = init_theory_set( sum(init_theory_set,2) < max_laws, :); 

theory_set=[];
theory_compliance={};
    Z_model_given_theory=zeros(1,length(UF));
    model_weights=cell(1,length(UF));
%theory_compliance = zeros(num_dags, size(init_theory_set,1), num_action_sets);
for t=1:size(init_theory_set,1)
    for u=1:length(UF)
    model_weights{u} = prod(rule_compliance{u}(:,:,init_theory_set(t,:)),3);
    Z_model_given_theory(u) = sum(sum(model_weights{u},1),2); %total weights of all models
    end
    if all(Z_model_given_theory >0)
        theory_set = [theory_set; init_theory_set(t,:)];
        for u=1:length(UF)
        theory_compliance{u}(:,size(theory_set,1),:) = model_weights{u}./Z_model_given_theory(u);
        end
    else
        %display(['rejected unstaisfiable theory ' num2str(init_theory_set(t,:)) '.']);
    end
end

display(['built ' num2str(size(theory_set,1)) ' satisfiable theories.']);
toc

if ~return_cell
theory_compliance=theory_compliance{1};
end

save(['compliance_' num2str(N)], 'theory_compliance');
save('theories', 'theory_set');
