function laws = law_check(dag, A_vec, UF, RF, disp)
%
% check the laws on a model (dag/action_set pair)...
% UF and RF are fixed background predicates (features and relational features of events)
%   UF is a (set of?) unary feature.
%   RF is a relational feature.
%
% Note: if we are using at most 4 laws per theory, and can handle a few thousand theories, then we can only do up to about 15 laws. 

if nargin<5
disp=0;
end

laws=[];

%%laws of the form: Q1 x in C1, Q2 y in C2, R-term(x,y)
Rxy = logical(dag);
Ryx = Rxy';
A=logical(A_vec');
UF=logical(UF);
Vars=logical(ones(length(A),1));
EQxy = eye(length(A));


%the first four laws are those for the true theory: 

desc='Actions are exogenous.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ \forall y\ A(x) \rightarrow \neg R(y,x)$ '...
        '& ' desc '\\']); end
laws(end+1) = all_x_in(A, all_y_in(Vars, ~Ryx));

desc='Actions have at most one child.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ A(x) \rightarrow \qf y\ R(x,y)$ '...   
        '& ' desc '\\']);end
laws(end+1) = all_x_in(A, zero_or_one_y_in(Vars, Rxy));

%%feature laws:
for i=1:size(UF,1)
    desc=['Fetaure ' num2str(i) ' is diagnostic for actions.'];
    if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ F_{' num2str(i) '}(x) \rightarrow A(x)$ '...
            '& ' desc '\\']);end
    laws(end+1) = all_x_in(UF(i,:)', A);
    
%     desc='';
%     if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ A(x) \rightarrow F_{' num2str(i) '}(x)$ '...
%             '& ' desc '\\']);end
%     laws(end+1) = all_x_in(A, UF(i,:)');
    
%     if disp display(['Law \#' num2str(1+length(laws)) ': & $\textsf{M}\ x\ F_{' num2str(i) '}(x) \rightarrow A(x)$ '...
% '& ' desc '\\']);end
%     laws(end+1) = most_x_in(UF(i,:)', A);
%         
%     if disp display(['Law \#' num2str(1+length(laws)) ': & $\textsf{M}\ x\ A(x) \rightarrow F_{' num2str(i) '}(x)$ '... 
% '& ' desc '\\']);end
%     laws(end+1) = most_x_in(A, UF(i,:)');
end

% desc='All actions have a child.';
% if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ A(x) \rightarrow \exists y\ R(x,y)$ '...
%         '& ' desc '\\']);end
% laws(end+1) = all_x_in(A, any_y_in(Vars, Rxy));


%%laws about the world

desc='Dag is fully connected.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x \ \forall y \ R(x,y) \vee R(y,x) \vee x{=}y $ '...
        '& ' desc '\\']);end
laws(end+1) = all_x_in(Vars, all_y_in(Vars, Rxy | Ryx | EQxy));

desc='Dag is unconnected.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ \forall y\ \neg R(x,y)$ '...
        '& ' desc '\\']); end
laws(end+1) = all_x_in(Vars, all_y_in(Vars, ~Rxy));

desc='Variables have at most one child.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ \qf y\ R(x,y)$ '...
        '& ' desc '\\']); end
laws(end+1) = all_x_in(Vars, zero_or_one_y_in(Vars, Rxy));

desc='Variables have at most one parent.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ \qf y\ R(y,x)$ '...
        '& ' desc '\\']); end
laws(end+1) = all_x_in(Vars, zero_or_one_y_in(Vars, Ryx));

desc='Causality is transitive.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ \forall y\ \forall z\ R(x,y) \wedge R(y,z) \rightarrow R(x,z)$ '...
        '& ' desc '\\']); end
laws(end+1) = all(all(~logical(dag*dag) | Rxy));

%%laws involving A
desc='Actions have no children.';
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ \forall y\ A(x) \rightarrow \neg R(x,y)$ '...
        '& ' desc '\\']); end
laws(end+1) = all_x_in(A, all_y_in(Vars, ~Rxy));

desc='Variables have at most one non-action parent.'; 
if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ \qf y\ \neg A(y) \wedge R(y,x)$ '...
        '& ' desc '\\']); end
laws(end+1) = all_x_in(Vars, zero_or_one_y_in(~A, Ryx));



% %a relation law
% desc=['Relation 1 is diagnostic for causation.'];
% if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x\ F_{' num2str(i) '}(x) \rightarrow A(x)$ '...
%         '& ' desc '\\']);end
% laws(end+1) = all(all((Rxy | ~logical(RF))));



return


% if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x {\in} A \ \forall y \ \neg R(y,x)$ '...         '& ' desc '\\']); end
% laws(end+1) = all_x_in(A, all_y_in(Vars, ~Ryx));
% 
% if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x \ \forall y \ \neg R(y,x)$ '...         '& ' desc '\\']);end
% laws(end+1) = all_x_in(Vars, all_y_in(Vars, ~Ryx));
% 
% if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x {\in} A \ \exists! y \ R(x,y)$ '...         '& ' desc '\\']);end
% laws(end+1) = all_x_in(A, one_y_in(Vars, Rxy));
% 
% if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x {\in} A \ \exists! y \ R(x,y)$ '...         '& ' desc '\\']);end
% laws(end+1) = all_x_in(A, one_y_in(Vars, Rxy));
% 
% if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x \ \forall y \ R(x,y) \wedge R(y,x) \wedge x{=}y $ '...         '& ' desc '\\']);end
% laws(end+1) = all_x_in(Vars, all_y_in(Vars, Rxy | Ryx | EQxy));


% %%feature laws:
% for i=1:size(UF,1)
% %     laws(end+1) = all_x_in(UF(i,:)', A);
% %     laws(end+1) = most_x_in(UF(i,:)', A);
%     if disp display(['Law \#' num2str(1+length(laws)) ': & $\forall x {\in} A \ F_{' num2str(i) '}(x)$ '...         '& ' desc '\\']);end
%     laws(end+1) = all_x_in(A, UF(i,:)');
%     if disp display(['Law \#' num2str(1+length(laws)) ': & $\textsf{M}\ x {\in} A \ F_{' num2str(i) '}(x)$ '...         '& ' desc '\\']);end
%     laws(end+1) = most_x_in(A, UF(i,:)');
% end



% return


%R-terms:
R_terms.mat={};
R_terms.desc={};
R_terms.mat{end+1} = Rxy;
R_terms.desc{end+1} = 'Rxy';
R_terms.mat{end+1} = Ryx;
R_terms.desc{end+1} = 'Ryx';
R_terms.mat{end+1} = ~Rxy;
R_terms.desc{end+1} = '~Rxy';
R_terms.mat{end+1} = ~Ryx;
R_terms.desc{end+1} = '~Ryx';
R_terms.mat{end+1} = Rxy | Ryx | EQxy;
R_terms.desc{end+1} = 'Rxy | Ryx | x=y';

%allowed classes:
Classes.mat={A, ~A, U};
Classes.desc={'A', 'not A', 'Vars'};

for r=1:length(R_terms.mat)
    R_term=R_terms.mat{r};
    
    for class1=1:length(Classes.mat)
        Cx=Classes.mat{class1};
        for class2=1:length(Classes.mat)
            Cy=Classes.mat{class2};
            
            display(['Law #' num2str(1+length(laws)) ': all x in ' Classes.desc{class1} ...
                ' all y in ' Classes.desc{class2} ...
                '  ' R_terms.desc{r}]);
            laws(end+1) = all_x_in(Cx, all_y_in(Cy, R_term));
            
            display(['Law #' num2str(1+length(laws)) ': all x in ' Classes.desc{class1} ...
                ' any y in ' Classes.desc{class2} ...
                '  ' R_terms.desc{r}]);
            laws(end+1) = all_x_in(Cx, any_y_in(Cy, R_term));
            
            display(['Law #' num2str(1+length(laws)) ': all x in ' Classes.desc{class1} ...
                ' one y in ' Classes.desc{class2} ...
                '  ' R_terms.desc{r}]);
            laws(end+1) = all_x_in(Cx, one_y_in(Cy, R_term));
            
            display(['Law #' num2str(1+length(laws)) ': most x in ' Classes.desc{class1} ...
                ' all y in ' Classes.desc{class2} ...
                '  ' R_terms.desc{r}]);
            laws(end+1) = most_x_in(Cx, all_y_in(Cy, R_term));
        end
    end
end


%%feature laws:
for i=1:size(UF,1)
    laws(end+1) = all_x_in(UF(i,:)', A);
    laws(end+1) = most_x_in(UF(i,:)', A);
    laws(end+1) = all_x_in(A, UF(i,:)');
    laws(end+1) = most_x_in(A, UF(i,:)');
end




%%generalized quantifiers

function c = all_x_in(A,T)
c = all( T(A,:), 1);

function c = any_x_in(A,T)
c = any( T(A,:), 1);

function c = most_x_in(A,T)
c = prod(T(A,:) + 0.2.*(~T(A,:)),1);

function c = one_x_in(A,T)
c = sum(T(A,:), 1)==1;



function c = all_y_in(A,T)
c = all( T(:,A), 2);

function c = any_y_in(A,T)
c = any( T(:,A), 2);

function c = one_y_in(A,T)
c = sum(T(:,A), 2)==1;

function c = zero_or_one_y_in(A,T)
c = sum(T(:,A), 2)<2;






% A_vec = logical(A_vec);
%
% num_vars = size(dag,1);
%
% A1 = repmat(A_vec, num_vars,1);
% A2 = A1';
%
% laws=[];

%%standard CBN learning assumes the first three laws.

% %actions have no parents.
% laws(end+1) = all(all( ~A1 | ~dag));
%
% %actions have exactly one child.
% laws(end+1) = all(sum(dag(A_vec,:),2)==1);

% %a variable is an action iff UF holds.
% laws(end+1) = all(A_vec == UF);
%
% %a variable is an action iff UF holds -- but softly.
% laws(end+1) = prod((A_vec == UF) + 0.2.*(A_vec ~= UF));

% %the dag fully connected.
% laws(end+1) = all(all(dag | dag' | eye(num_vars)))  ;

