
function scores = score_all(dags, systems_and_samples, parameters)

% This function computes the score for each dag (row) and each system (column)
% for each prefix of samples seen (third dimension).
%
% systems_and_samples is a matrix of samples X variables X systems
%
% returns a score matrix: dags X systems X data_prefix 

num_of_dags = length(dags);
num_of_data_sets = length(parameters.score_data_at);
num_of_nodes = size(systems_and_samples,2);
num_of_systems = size(systems_and_samples,3);

num_of_samples=size(systems_and_samples,1);

scores = zeros(num_of_dags, num_of_systems, num_of_data_sets);

count_score_cache=spalloc(num_of_samples+1,num_of_samples+1, num_of_samples);

% We score the dags using the cache function, which is reset for every
% time we change the sample base.

for k = 1:num_of_systems
    
    %for i = 1:num_of_samples
        
        cache = [];
        
        for j = 1:num_of_dags
            
            dag = ind2dg(dags(j), num_of_nodes);
            
            [scores(j,k,:), cache, count_score_cache] = score_using_cache(dag,systems_and_samples(:,:,k),parameters, cache, count_score_cache);
            
        end
        
    %end
    
end



