function [theory_log_prob log_prob_dag_given_data_theory log_prob_dag_given_data] = ...
    theory_log_posterior(theory_set, data_log_prob, theory_compliance, data_set_per_system, theories_of_interest)

% data_log_prob  is P(D_s | R_s)  (for varying amounts of data):  
%           dags X systems X data_set
%
% rows of data_set_per_system indicate which data set should be used for
% each system (in order to model non-uniform evidence distribution).

if nargin>3
    %munge up data_log_prob to put them where needed by
    %which_data_set_for_each_system.....
    tmp=zeros(size(data_log_prob,1), size(data_log_prob,2), size(data_set_per_system,1));
    for s=1:size(data_set_per_system,2)
        tmp(:,s,:) = data_log_prob(:,s,data_set_per_system(:,s));
    end
    data_log_prob=tmp;
end


num_dags = size(data_log_prob,1);
num_systems = size(data_log_prob,2);
num_data_sets = size(data_log_prob,3);
num_theories = size(theory_set,1);

if nargin<5
    theories_of_interest=1:num_theories;
end
num_theories_of_interest=length(theories_of_interest);


%these things don't depend on data set, so can be done outside the data_set
%loop:
%P(R_s, A_s | T):
model_log_priors = log(theory_compliance); %this is normalized in build_theory.
%P(R_s | T), marginalize over the action set (logsumexp over A, gives vector size of dags)
%is dags X theories
dag_log_priors = logsumexp(model_log_priors,3);
%replicate across systems, and reorder to be (dags X systems X theories)
dag_log_priors = permute(repmat(dag_log_priors, [1 1 num_systems]), [1 3 2]);


theory_log_prob = zeros(num_theories,num_data_sets);
log_prob_dag_given_data_specific_theory=zeros(num_dags, num_systems, num_theories_of_interest, num_data_sets);
log_prob_dag_given_data=zeros(num_dags, num_systems, num_data_sets);
for e=1:num_data_sets
   
    %(dags X systems X theories)
    score_dag_given_theory_data = repmat(data_log_prob(:,:,e), [1 1 num_theories]) + dag_log_priors;
    
    %now marginalize out the dags:
    %(1 X systems X theories)
    system_score = logsumexp( score_dag_given_theory_data, 1);
        
    %finally sum over theories and normalize:
    %(1 X 1 X theories)
    theory_scores=sum(system_score,2);
    theory_scores=theory_scores - repmat(logsumexp(theory_scores,3), [1 1 num_theories]);
    theory_log_prob(:,e) = theory_scores;
    
    %P(R_s | D_s, T) \propto  P(D_s | R_s) P(R_s | T)
    %(dags X systems X theories)
    log_prob_dag_given_data_theory = score_dag_given_theory_data - repmat(system_score, [num_dags 1 1]);
    
    %dag marginal posterior (marginal over theories),
    % P(R_s | D) = sum_T P(R_s | D_s, T) P(T | D)
    th=repmat(theory_scores, [num_dags num_systems 1]); %(dags X systems X theories)
    log_prob_dag_given_data(:,:,e) = logsumexp( log_prob_dag_given_data_theory + th , 3);
    
    log_prob_dag_given_data_specific_theory(:,:,:,e)=log_prob_dag_given_data_theory(:,:,theories_of_interest);
end
        
log_prob_dag_given_data_theory=permute(log_prob_dag_given_data_specific_theory, [1 2 4 3]);

